package sun.security.krb5.internal.tools;

import sun.security.krb5.internal.ccache.CCacheInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import sun.security.krb5.KrbException;
import sun.security.krb5.RealmException;
import sun.security.krb5.internal.ccache.FileCredentialsCache;
import sun.security.krb5.internal.Krb5;
import sun.security.krb5.PrincipalName;
import sun.security.krb5.internal.KerberosTime;

class KinitOptions
{
    public boolean validate;
    public short forwardable;
    public short proxiable;
    public boolean renew;
    public KerberosTime lifetime;
    public KerberosTime renewable_lifetime;
    public String target_service;
    public String keytab_file;
    public String cachename;
    private PrincipalName principal;
    public String realm;
    char[] password;
    public boolean keytab;
    private boolean DEBUG;
    private boolean includeAddresses;
    private boolean useKeytab;
    private String ktabName;
    
    public KinitOptions() throws RuntimeException, RealmException {
        this.validate = false;
        this.forwardable = -1;
        this.proxiable = -1;
        this.renew = false;
        this.password = null;
        this.DEBUG = Krb5.DEBUG;
        this.includeAddresses = true;
        this.useKeytab = false;
        this.cachename = FileCredentialsCache.getDefaultCacheName();
        if (this.cachename == null) {
            throw new RuntimeException("default cache name error");
        }
        this.principal = this.getDefaultPrincipal();
        return;
    }
    
    public void setKDCRealm(String realm) throws RealmException {
        this.realm = realm;
    }
    
    public String getKDCRealm() {
        if (this.realm == null && this.principal != null) {
            return this.principal.getRealmString();
        }
        return null;
    }
    
    public KinitOptions(String[] array) throws KrbException, RuntimeException, IOException {
        this.validate = false;
        this.forwardable = -1;
        this.proxiable = -1;
        this.renew = false;
        this.password = null;
        this.DEBUG = Krb5.DEBUG;
        this.includeAddresses = true;
        this.useKeytab = false;
        String s = null;
        for (int i = 0; i < array.length; ++i) {
            if (array[i].equals("-f")) {
                this.forwardable = 1;
            }
            else if (array[i].equals("-p")) {
                this.proxiable = 1;
            }
            else if (array[i].equals("-c")) {
                if (array[i + 1].startsWith("-")) {
                    throw new IllegalArgumentException("input format  not correct:  -c  option must be followed by the cache name");
                }
                this.cachename = array[++i];
                if (this.cachename.length() >= 5 && this.cachename.substring(0, 5).equalsIgnoreCase("FILE:")) {
                    this.cachename = this.cachename.substring(5);
                }
            }
            else if (array[i].equals("-A")) {
                this.includeAddresses = false;
            }
            else if (array[i].equals("-k")) {
                this.useKeytab = true;
            }
            else if (array[i].equals("-t")) {
                if (this.ktabName != null) {
                    throw new IllegalArgumentException("-t option/keytab file name repeated");
                }
                if (i + 1 < array.length) {
                    this.ktabName = array[++i];
                    this.useKeytab = true;
                }
                else {
                    throw new IllegalArgumentException("-t option requires keytab file name");
                }
            }
            else if (array[i].equalsIgnoreCase("-help")) {
                this.printHelp();
                System.exit(0);
            }
            else if (s == null) {
                s = array[i];
                try {
                    this.principal = new PrincipalName(s);
                }
                catch (Exception ex) {
                    throw new IllegalArgumentException("invalid Principal name: " + s + ex.getMessage());
                }
            }
            else if (this.password == null) {
                this.password = array[i].toCharArray();
            }
            else {
                throw new IllegalArgumentException("too many parameters");
            }
        }
        if (this.cachename == null) {
            this.cachename = FileCredentialsCache.getDefaultCacheName();
            if (this.cachename == null) {
                throw new RuntimeException("default cache name error");
            }
        }
        if (this.principal == null) {
            this.principal = this.getDefaultPrincipal();
        }
    }
    
    PrincipalName getDefaultPrincipal() {
        try {
            CCacheInputStream cCacheInputStream = new CCacheInputStream(new FileInputStream(this.cachename));
            int version;
            if ((version = cCacheInputStream.readVersion()) == 1284) {
                cCacheInputStream.readTag();
            }
            else if (version == 1281 || version == 1282) {
                cCacheInputStream.setNativeByteOrder();
            }
            PrincipalName principal = cCacheInputStream.readPrincipal(version);
            cCacheInputStream.close();
            if (this.DEBUG) {
                System.out.println(">>>KinitOptions principal name from the cache is :" + principal);
                return principal;
            }
            return principal;
        }
        catch (IOException ex) {
            if (this.DEBUG) {
                ex.printStackTrace();
            }
        }
        catch (RealmException ex2) {
            if (this.DEBUG) {
                ex2.printStackTrace();
            }
        }
        String property = System.getProperty("user.name");
        if (this.DEBUG) {
            System.out.println(">>>KinitOptions default username is :" + property);
        }
        try {
            return new PrincipalName(property);
        }
        catch (RealmException ex3) {
            if (this.DEBUG) {
                System.out.println("Exception in getting principal name " + ex3.getMessage());
                ex3.printStackTrace();
            }
            return null;
        }
    }
    
    void printHelp() {
        System.out.println("Usage: kinit [-A] [-f] [-p] [-c cachename] [[-k [-t keytab_file_name]] [principal] [password]");
        System.out.println("\tavailable options to Kerberos 5 ticket request:");
        System.out.println("\t    -A   do not include addresses");
        System.out.println("\t    -f   forwardable");
        System.out.println("\t    -p   proxiable");
        System.out.println("\t    -c   cache name (i.e., FILE:\\d:\\myProfiles\\mykrb5cache)");
        System.out.println("\t    -k   use keytab");
        System.out.println("\t    -t   keytab file name");
        System.out.println("\t    principal   the principal name (i.e., qweadf@ATHENA.MIT.EDU qweadf)");
        System.out.println("\t    password   the principal's Kerberos password");
    }
    
    public boolean getAddressOption() {
        return this.includeAddresses;
    }
    
    public boolean useKeytabFile() {
        return this.useKeytab;
    }
    
    public String keytabFileName() {
        return this.ktabName;
    }
    
    public PrincipalName getPrincipal() {
        return this.principal;
    }
}
